body {
 font-family: Verdana, Arial, Helvetica, sans-serif;
 font-size: 14px;
 font-weight: normal;
 margin: 0px;
 padding: 0px;
}

#dependencies {
  font-size: 12px;
  
}
#dependencies li {
  display:   inline;
}

#special_files li {
  display:   inline;
}

#content {
  float: left;
  width: 700px;
  margin-left: 20px;
/*  background-color: yellow; */
}

#right_column {
  float: right;
  width: 300px;
/*  background-color: red; */
}


#menu ul li {
   display: inline;
   font-size: 18px;
   margin-left: 10px;
}

#ellapsed_time {
}

/* syntax highlighting */
.code a {
  text-decoration: none;
}

.code a:hover {
  text-decoration: underline;
}

.keyword {
  color: green;
  display:   inline;
}

.ws {
  display:   inline;
}

.Whitespace {
  display:   inline;
}

.Word {
  color: #232323;
  display:   inline;
}
.pragma {
  display:   inline;
}
.Single {
  display:   inline;
}
.Float {
  display:   inline;
}
.Symbol {
  display:   inline;
}
.Operator {
  display:   inline;
}
.Structure {
  display:   inline;
  color: green;
}

.Pod {
  display:   inline;
  color: grey;
}


.Comment {
  color: grey;
  display:   inline;
}

.ArrayIndex {
  display:   inline;
}

.Label {
  display:   inline;
}

.Separator {
  display:   inline;
}

.Cast {
  display:   inline;
}

.Number {
  display:   inline;
}

.core {
  display:   inline;
}

.Magic {
  display:   inline;
}

.Words {
  display:   inline;
}

.row {
  color: blue;
  display:   block;
}

